/* Gtick  -  A Gtk+ Metronome
 *
 *   main.c: Main app code
 *
 * Copyright (c) 1999, Alex Roberts
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <glib.h>
#include <stdio.h>
#include "gtick.h"

TikTok *gtick;
static int running = 0;
int rate;
  
void gtQuit (GtkWidget *w, GtkWidget *window)
{

	gtk_widget_destroy (window);
	gtk_main_quit ();
}

gint gtTick (gpointer data)
{
	gtDSPTick();
	
	return TRUE;
}

void gtUpdateRate (TikTok *tick, gpointer data)
{
TikTok *ticker = (TikTok *)data;

	if (ticker->ticking)
	  {
	    gtk_timeout_remove (running);
	    running = 0;
	  }	    
	    
	    rate = 1000/((GTK_ADJUSTMENT(ticker->adjustment)->value)/60.0);

#ifdef DEBUG	     
	    g_print ("gtUpdateRate: rate=%f running at %d ms\n",
	      			(GTK_ADJUSTMENT(ticker->adjustment)->value),rate);
#endif

	if (ticker->ticking)
	  {
	    running = gtk_timeout_add (rate, (GtkFunction) gtTick, NULL);
	  }	

	    
}

void gtBeat (TikTok *tick, gpointer data)
{
TikTok *ticker = (TikTok *)data;
  
	 if (!ticker->ticking)
	   {
	     gtk_frame_set_label (GTK_FRAME(ticker->box3), "Ticker: On");
	     ticker->ticking = TRUE;
	     /* Do Metronome stuff */
	     
	     running = gtk_timeout_add (rate, (GtkFunction) gtTick, NULL);
	   }
	else
	   {
	     gtk_frame_set_label (GTK_FRAME(ticker->box3), "Ticker: Off");	  
	     ticker->ticking = FALSE;
	     /* Stop all Metronome stuff */
	     /*gtCloseDSP();*/
	     gtk_timeout_remove (running);
	     running = 0;
	   }
}

void gtInit (TikTok *ticker)
{
      ticker->window = gtk_window_new (GTK_WINDOW_TOPLEVEL);

      gtk_signal_connect (GTK_OBJECT (ticker->window), "destroy",
			  GTK_SIGNAL_FUNC(gtQuit),
			  &ticker->window);

      gtk_window_set_title (GTK_WINDOW (ticker->window),TITLE);
      gtk_container_set_border_width (GTK_CONTAINER (ticker->window), 5);


      ticker->box1 = gtk_vbox_new (FALSE, 5);
      gtk_container_add (GTK_CONTAINER (ticker->window), ticker->box1);
      gtk_widget_show (ticker->box1);

	ticker->box3 = gtk_frame_new ("Ticker: Off");
	gtk_container_add (GTK_CONTAINER (ticker->box1), ticker->box3);
      gtk_widget_show (ticker->box3);

      ticker->box2 = gtk_hbox_new (FALSE, 10);
      gtk_container_set_border_width (GTK_CONTAINER (ticker->box2), 10);
      /*gtk_box_pack_start (GTK_BOX (ticker->box1), ticker->box2, TRUE, TRUE, 0);*/
      gtk_container_add (GTK_CONTAINER (ticker->box3), ticker->box2);
      gtk_widget_show (ticker->box2);

	ticker->label = gtk_label_new ("BPM:");
	gtk_box_pack_start (GTK_BOX (ticker->box2), ticker->label, TRUE, TRUE, 0);
	gtk_widget_show (ticker->label);

      ticker->adjustment = gtk_adjustment_new (75.0, 30.0, 251.0, 1.0, 1.0, 1.0);
      gtk_signal_connect (GTK_OBJECT (ticker->adjustment), "value_changed",
                            GTK_SIGNAL_FUNC (gtUpdateRate), (TikTok *)ticker);
                            
	gtUpdateRate (ticker,(TikTok *)ticker); /* Set initial rate */

      ticker->scale = gtk_hscale_new (GTK_ADJUSTMENT (ticker->adjustment));
      gtk_widget_set_usize (GTK_WIDGET (ticker->scale), 150, 30);
      gtk_range_set_update_policy (GTK_RANGE (ticker->scale), GTK_UPDATE_DELAYED);
      gtk_scale_set_digits (GTK_SCALE (ticker->scale), 1);
      gtk_scale_set_draw_value (GTK_SCALE (ticker->scale), TRUE);
      gtk_box_pack_start (GTK_BOX (ticker->box2), ticker->scale, TRUE, TRUE, 0);
      gtk_widget_show (ticker->scale);


      ticker->separator = gtk_hseparator_new ();
      gtk_box_pack_start (GTK_BOX (ticker->box1), ticker->separator, FALSE, TRUE, 0);
      gtk_widget_show (ticker->separator);


      ticker->box2 = gtk_hbox_new (FALSE, 5);
      gtk_container_set_border_width (GTK_CONTAINER (ticker->box2), 5);
      gtk_box_pack_start (GTK_BOX (ticker->box1), ticker->box2, FALSE, TRUE, 0);
      gtk_widget_show (ticker->box2);


      ticker->togglebtn = gtk_button_new_with_label ("Toggle Ticker");
      gtk_signal_connect (GTK_OBJECT (ticker->togglebtn), "clicked",
				 GTK_SIGNAL_FUNC(gtBeat),
				 (TikTok *)ticker);
      gtk_box_pack_start (GTK_BOX (ticker->box2), ticker->togglebtn, TRUE, TRUE, 0);
      GTK_WIDGET_SET_FLAGS (ticker->togglebtn, GTK_CAN_DEFAULT);
      gtk_widget_grab_default (ticker->togglebtn);
      gtk_widget_show (ticker->togglebtn);

      ticker->button = gtk_button_new_with_label ("Quit");
      gtk_signal_connect_object (GTK_OBJECT (ticker->button), "clicked",
				 GTK_SIGNAL_FUNC(gtQuit),
				 GTK_OBJECT(ticker->window));
      gtk_box_pack_start (GTK_BOX (ticker->box2), ticker->button, TRUE, TRUE, 0);
      GTK_WIDGET_SET_FLAGS (ticker->button, GTK_CAN_DEFAULT);
      gtk_widget_show (ticker->button);
}

int main()
{

	gtk_init(NULL,NULL);

	gtick = g_malloc (sizeof (TikTok));
	
	gtInit (gtick);
        gtInitDSP();

      	gtk_widget_show (gtick->window);
      	
      gtk_main();
      
      gtCloseDSP();
      return 0;
}